#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import UNOFFICIAL, WORKDAY
from holidays.countries.finland import Finland
from tests.common import CommonCountryTests


class TestFinland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Finland)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Finland(categories=UNOFFICIAL, years=range(self.start_year, 1950)))
        self.assertNoHolidays(Finland(categories=WORKDAY, years=range(self.start_year, 1919)))

    def test_epiphany(self):
        name = "Loppiainen"
        self.assertHolidayName(
            name,
            "1972-01-06",
            # 1973 - 1990 changes.
            "1973-01-06",
            "1974-01-12",
            "1989-01-07",
            "1990-01-06",
            # 1991 - Present.
            "1991-01-06",
        )
        self.assertNoHolidayName(
            name,
            "1974-01-06",
            "1975-01-06",
            "1980-01-06",
            "1988-01-06",
            "1989-01-06",
        )
        self.assertHolidayName(
            name,
            (
                f"{year}-01-06"
                for year in (*range(self.start_year, 1973), *range(1991, self.end_year))
            ),
        )
        self.assertHolidayName(name, range(1973, 1991))

    def test_good_friday(self):
        name = "Pitkäperjantai"
        self.assertHolidayName(
            name,
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Pääsiäispäivä"
        self.assertHolidayName(
            name,
            "2018-04-01",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Toinen pääsiäispäivä"
        self.assertHolidayName(
            name,
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )
        self.assertHolidayName(name, self.full_range)

    def test_ascension_day(self):
        name = "Helatorstai"
        self.assertHolidayName(
            name,
            "1971-05-20",
            "1972-05-11",
            # 1973 - 1990 changes.
            "1973-05-26",
            "1974-05-18",
            "1989-04-29",
            "1990-05-19",
            # 1991 - Present.
            "1991-05-09",
            "1992-05-28",
            "1993-05-20",
            "2018-05-10",
            "2019-05-30",
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
        )
        self.assertNoHolidayName(
            name,
            "1973-05-31",
            "1974-05-23",
            "1980-05-15",
            "1988-05-12",
            "1989-05-04",
            "1990-05-24",
        )
        self.assertHolidayName(name, self.full_range)

    def test_may_day(self):
        name = "Vappu"
        # PUBLIC.
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1944, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1944))
        # WORKDAY.
        self.assertWorkdayHolidayName(
            name, (f"{year}-05-01" for year in range(1979, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1979))

    def test_whit_sunday(self):
        name = "Helluntaipäivä"
        self.assertHolidayName(
            name,
            "2018-05-20",
            "2019-06-09",
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
        )
        self.assertHolidayName(name, self.full_range)

    def test_midsummer_eve(self):
        name = "Juhannusaatto"
        self.assertHolidayName(
            name,
            "1953-06-23",
            "1954-06-23",
            "1955-06-24",
            # 1955 - Present.
            "1956-06-22",
            "1957-06-21",
        )
        self.assertNoHolidayName(
            name,
            "1955-06-23",
            "1956-06-23",
            "1957-06-23",
        )
        self.assertHolidayName(name, (f"{year}-06-23" for year in range(self.start_year, 1955)))
        self.assertHolidayName(name, range(1955, self.end_year))

    def test_midsummer_day(self):
        name = "Juhannuspäivä"
        name_workday = "Suomen lipun päivä"
        dt = (
            "1953-06-24",
            "1954-06-24",
            # 1955 - Present.
            "1955-06-25",
            "1956-06-23",
            "1957-06-22",
        )
        dt_no_obs = (
            "1955-06-24",
            "1956-06-24",
            "1957-06-24",
        )
        # PUBLIC.
        self.assertHolidayName(name, dt)
        self.assertNoHolidayName(name, dt_no_obs)
        self.assertHolidayName(name, (f"{year}-06-24" for year in range(self.start_year, 1955)))
        self.assertHolidayName(name, range(1955, self.end_year))
        # WORKDAY.
        self.assertNoHolidayName(name_workday)
        self.assertWorkdayHolidayName(name_workday, dt)
        self.assertNoWorkdayHolidayName(name_workday, dt_no_obs)
        self.assertWorkdayHolidayName(
            name_workday, (f"{year}-06-24" for year in range(1934, 1955))
        )
        self.assertWorkdayHolidayName(name_workday, range(1955, self.end_year))
        self.assertNoWorkdayHolidayName(name_workday, range(self.start_year, 1934))

    def test_all_saints_day(self):
        name = "Pyhäinpäivä"
        self.assertHolidayName(
            name,
            "1952-11-01",
            "1953-11-01",
            "1954-11-01",
            # 1955 - Present.
            "1955-11-05",
            "1956-11-03",
            "1957-11-02",
        )
        self.assertNoHolidayName(
            name,
            "1955-11-01",
            "1956-11-01",
            "1957-11-01",
        )
        self.assertHolidayName(name, (f"{year}-11-01" for year in range(self.start_year, 1955)))
        self.assertHolidayName(name, range(1955, self.end_year))

    def test_independence_day(self):
        name = "Itsenäisyyspäivä"
        # PUBLIC.
        self.assertHolidayName(name, (f"{year}-12-06" for year in range(1919, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1919))
        # WORKDAY.
        self.assertWorkdayHolidayName(
            name, (f"{year}-12-06" for year in range(1919, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1919))

    def _test_categorized_holiday(self, name, holidays_list, since, end_year=None):
        if end_year is None:
            end_year = self.end_year
        start_year, month, day = (int(p) for p in since.split("-"))
        self.assertNoHolidayName(name)
        self.assertHolidayName(
            name,
            holidays_list,
            (f"{year}-{month:02d}-{day:02d}" for year in range(start_year, end_year)),
        )
        self.assertNoHolidayName(name, holidays_list, range(self.start_year, start_year))
        if end_year != self.end_year:
            self.assertNoHolidayName(name, holidays_list, range(end_year, self.end_year))

    def _test_unofficial_holiday(self, name, since, end_year=None):
        self._test_categorized_holiday(name, self.holidays_unofficial, since, end_year)

    def _test_workday_holiday(self, name, since, end_year=None):
        self._test_categorized_holiday(name, self.holidays_workday, since, end_year)

    def test_runeberg_day(self):
        self._test_unofficial_holiday("Runebergin päivä", "1976-02-05")

    def test_kalevala_day_day_of_finnish_culture(self):
        self._test_workday_holiday("Kalevalan päivä, suomalaisen kulttuurin päivä", "1920-02-28")

    def test_minna_canth_day_day_of_equality(self):
        self._test_unofficial_holiday("Minna Canthin päivä, tasa-arvon päivä", "2007-03-19")

    def test_mikael_agricola_day_day_of_finnish_language(self):
        self._test_unofficial_holiday("Mikael Agricolan päivä, suomen kielen päivä", "1980-04-09")

    def test_national_war_veterans_day(self):
        self._test_unofficial_holiday("Kansallinen veteraanipäivä", "1987-04-27")

    def test_europe_day(self):
        self._test_unofficial_holiday("Eurooppa-päivä", "2019-05-09")

    def test_mothers_day(self):
        name = "Äitienpäivä"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "1947-05-11",
            "1948-05-09",
            "2020-05-10",
            "2024-05-12",
        )
        self.assertWorkdayHolidayName(name, range(1947, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1947))

    def test_j_v_snellmans_day_day_of_finnish_heritage(self):
        self._test_unofficial_holiday("J.V. Snellmanin päivä", "1952-05-12", 1978)
        self._test_unofficial_holiday("J.V. Snellmanin päivä, suomalaisuuden päivä", "1978-05-12")

    def test_remembrance_day(self):
        name = "Kaatuneitten muistopäivä"
        self.assertNoHolidayName(name)
        self.assertUnofficialHolidayName(
            name,
            "1977-05-15",
            "1978-05-21",
            "1985-05-19",
            "2024-05-19",
            "2025-05-18",
        )
        self.assertUnofficialHolidayName(name, range(1977, self.end_year))
        self.assertNoUnofficialHolidayName(name, range(self.start_year, 1977))

    def test_flag_day_of_the_finnish_defense_forces(self):
        self._test_workday_holiday("Suomen marsalkan syntymäpäivä", "1942-06-06", 1950)
        self._test_workday_holiday("Puolustusvoimain lippujuhlan päivä", "1950-06-06")

    def test_eino_leino_day_day_of_summer_and_poetry(self):
        self._test_unofficial_holiday("Eino Leinon päivä, runon ja suven päivä", "1998-07-06")

    def test_finlands_nature_day(self):
        name = "Suomen luonnon päivä"
        self.assertNoHolidayName(name)
        self.assertUnofficialHolidayName(
            name,
            "2023-08-26",
            "2024-08-31",
            "2025-08-30",
            "2026-08-29",
        )
        self.assertUnofficialHolidayName(name, range(2023, self.end_year))
        self.assertNoUnofficialHolidayName(name, range(self.start_year, 2023))

    def test_miina_sillanpaa_day_day_of_civic_participation(self):
        self._test_unofficial_holiday(
            "Miina Sillanpään ja kansalaisvaikuttamisen päivä", "2016-10-01"
        )

    def test_aleksis_kivi_day_day_of_finnish_literature(self):
        self._test_unofficial_holiday("Aleksis Kiven päivä", "1950-10-10", 1978)
        self._test_unofficial_holiday(
            "Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä", "1978-10-10"
        )

    def test_united_nations_day(self):
        self._test_unofficial_holiday("YK:n päivä", "1987-10-24")

    def test_finnish_swedish_heritage_day_svenska_dagen(self):
        self._test_unofficial_holiday("Ruotsalaisuuden päivä, Kustaa Aadolfin päivä", "1979-11-06")

    def test_fathers_day(self):
        name = "Isänpäivä"
        self.assertNoHolidayName(name)
        # UNOFFICIAL.
        self.assertUnofficialHolidayName(
            name,
            "1987-11-08",
            "1988-11-13",
            "2017-11-12",
            "2018-11-11",
        )
        self.assertUnofficialHolidayName(name, range(1987, 2019))
        self.assertNoUnofficialHolidayName(
            name, range(self.start_year, 1987), range(2019, self.end_year)
        )
        # WORKDAY.
        self.assertWorkdayHolidayName(
            name,
            "2019-11-10",
            "2020-11-08",
            "2021-11-14",
            "2024-11-10",
        )
        self.assertWorkdayHolidayName(name, range(2019, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2019))

    def test_day_of_childrens_rights(self):
        self._test_unofficial_holiday("Lapsen oikeuksien päivä", "2020-11-20")

    def test_jean_sibelius_day_day_of_finnish_music(self):
        self._test_unofficial_holiday(
            "Jean Sibeliuksen päivä, suomalaisen musiikin päivä", "2011-12-08"
        )

    def test_alands_autonomy_day(self):
        name = "Ahvenanmaan itsehallintopäivä"
        self.assertNoHolidayName(name)
        self.assertSubdiv01HolidayName(
            name, (f"{year}-06-09" for year in range(1993, self.end_year))
        )
        self.assertNoSubdiv01HolidayName(name, range(self.start_year, 1993))

    def test_unofficial_holidays(self):
        self.assertUnofficialHolidaysInYear(
            2024,
            ("2024-02-05", "Runebergin päivä"),
            ("2024-03-19", "Minna Canthin päivä, tasa-arvon päivä"),
            ("2024-04-09", "Mikael Agricolan päivä, suomen kielen päivä"),
            ("2024-04-27", "Kansallinen veteraanipäivä"),
            ("2024-05-09", "Eurooppa-päivä"),
            ("2024-05-12", "J.V. Snellmanin päivä, suomalaisuuden päivä"),
            ("2024-05-19", "Kaatuneitten muistopäivä"),
            ("2024-07-06", "Eino Leinon päivä, runon ja suven päivä"),
            ("2024-08-31", "Suomen luonnon päivä"),
            ("2024-10-01", "Miina Sillanpään ja kansalaisvaikuttamisen päivä"),
            ("2024-10-10", "Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä"),
            ("2024-10-24", "YK:n päivä"),
            ("2024-11-06", "Ruotsalaisuuden päivä, Kustaa Aadolfin päivä"),
            ("2024-11-20", "Lapsen oikeuksien päivä"),
            ("2024-12-08", "Jean Sibeliuksen päivä, suomalaisen musiikin päivä"),
        )

    def test_workday_holidays(self):
        self.assertWorkdayHolidaysInYear(
            2024,
            ("2024-02-28", "Kalevalan päivä, suomalaisen kulttuurin päivä"),
            ("2024-05-01", "Vappu"),
            ("2024-05-12", "Äitienpäivä"),
            ("2024-06-06", "Puolustusvoimain lippujuhlan päivä"),
            ("2024-06-22", "Suomen lipun päivä"),
            ("2024-11-10", "Isänpäivä"),
            ("2024-12-06", "Itsenäisyyspäivä"),
        )

    def test_2018(self):
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "Uudenvuodenpäivä"),
            ("2018-01-06", "Loppiainen"),
            ("2018-03-30", "Pitkäperjantai"),
            ("2018-04-01", "Pääsiäispäivä"),
            ("2018-04-02", "Toinen pääsiäispäivä"),
            ("2018-05-01", "Vappu"),
            ("2018-05-10", "Helatorstai"),
            ("2018-05-20", "Helluntaipäivä"),
            ("2018-06-22", "Juhannusaatto"),
            ("2018-06-23", "Juhannuspäivä"),
            ("2018-11-03", "Pyhäinpäivä"),
            ("2018-12-06", "Itsenäisyyspäivä"),
            ("2018-12-24", "Jouluaatto"),
            ("2018-12-25", "Joulupäivä"),
            ("2018-12-26", "Tapaninpäivä"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Uudenvuodenpäivä"),
            ("2022-01-06", "Loppiainen"),
            ("2022-04-15", "Pitkäperjantai"),
            ("2022-04-17", "Pääsiäispäivä"),
            ("2022-04-18", "Toinen pääsiäispäivä"),
            ("2022-05-01", "Vappu"),
            ("2022-05-26", "Helatorstai"),
            ("2022-06-05", "Helluntaipäivä"),
            ("2022-06-24", "Juhannusaatto"),
            ("2022-06-25", "Juhannuspäivä"),
            ("2022-11-05", "Pyhäinpäivä"),
            ("2022-12-06", "Itsenäisyyspäivä"),
            ("2022-12-24", "Jouluaatto"),
            ("2022-12-25", "Joulupäivä"),
            ("2022-12-26", "Tapaninpäivä"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Uudenvuodenpäivä"),
            ("2024-01-06", "Loppiainen"),
            ("2024-02-05", "Runebergin päivä"),
            ("2024-02-28", "Kalevalan päivä, suomalaisen kulttuurin päivä"),
            ("2024-03-19", "Minna Canthin päivä, tasa-arvon päivä"),
            ("2024-03-29", "Pitkäperjantai"),
            ("2024-03-31", "Pääsiäispäivä"),
            ("2024-04-01", "Toinen pääsiäispäivä"),
            ("2024-04-09", "Mikael Agricolan päivä, suomen kielen päivä"),
            ("2024-04-27", "Kansallinen veteraanipäivä"),
            ("2024-05-01", "Vappu"),
            ("2024-05-09", "Eurooppa-päivä; Helatorstai"),
            ("2024-05-12", "J.V. Snellmanin päivä, suomalaisuuden päivä; Äitienpäivä"),
            ("2024-05-19", "Helluntaipäivä; Kaatuneitten muistopäivä"),
            ("2024-06-06", "Puolustusvoimain lippujuhlan päivä"),
            ("2024-06-09", "Ahvenanmaan itsehallintopäivä"),
            ("2024-06-21", "Juhannusaatto"),
            ("2024-06-22", "Juhannuspäivä; Suomen lipun päivä"),
            ("2024-07-06", "Eino Leinon päivä, runon ja suven päivä"),
            ("2024-08-31", "Suomen luonnon päivä"),
            ("2024-10-01", "Miina Sillanpään ja kansalaisvaikuttamisen päivä"),
            ("2024-10-10", "Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä"),
            ("2024-10-24", "YK:n päivä"),
            ("2024-11-02", "Pyhäinpäivä"),
            ("2024-11-06", "Ruotsalaisuuden päivä, Kustaa Aadolfin päivä"),
            ("2024-11-10", "Isänpäivä"),
            ("2024-11-20", "Lapsen oikeuksien päivä"),
            ("2024-12-06", "Itsenäisyyspäivä"),
            ("2024-12-08", "Jean Sibeliuksen päivä, suomalaisen musiikin päivä"),
            ("2024-12-24", "Jouluaatto"),
            ("2024-12-25", "Joulupäivä"),
            ("2024-12-26", "Tapaninpäivä"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-06", "Epiphany"),
            ("2024-02-05", "Runeberg Day"),
            ("2024-02-28", "Kalevala Day, Day of Finnish Culture"),
            ("2024-03-19", "Minna Canth Day, Day of Equality"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-31", "Easter Sunday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-09", "Mikael Agricola Day, Day of the Finnish Language"),
            ("2024-04-27", "National War Veterans' Day"),
            ("2024-05-01", "May Day"),
            ("2024-05-09", "Ascension Day; Europe Day"),
            ("2024-05-12", "J. V. Snellman Day, Day of Finnish Heritage; Mother's Day"),
            ("2024-05-19", "Remembrance Day; Whit Sunday"),
            ("2024-06-06", "Flag Day of the Finnish Defense Forces"),
            ("2024-06-09", "Åland's Autonomy Day"),
            ("2024-06-21", "Midsummer Eve"),
            ("2024-06-22", "Day of the Finnish Flag; Midsummer Day"),
            ("2024-07-06", "Eino Leino Day, Day of Summer and Poetry"),
            ("2024-08-31", "Finland's Nature Day"),
            ("2024-10-01", "Miina Sillanpää Day, Day of Civic Participation"),
            ("2024-10-10", "Aleksis Kivi Day, Day of Finnish Literature"),
            ("2024-10-24", "United Nations Day"),
            ("2024-11-02", "All Saints' Day"),
            ("2024-11-06", "Finnish Swedish Heritage Day, svenska dagen"),
            ("2024-11-10", "Father's Day"),
            ("2024-11-20", "Day of Children's Rights"),
            ("2024-12-06", "Independence Day"),
            ("2024-12-08", "Jean Sibelius Day, Day of Finnish Music"),
            ("2024-12-24", "Christmas Eve"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Second Day of Christmas"),
        )

    def test_l10n_sv_fi(self):
        self.assertLocalizedHolidays(
            "sv_FI",
            ("2024-01-01", "Nyårsdagen"),
            ("2024-01-06", "Trettondag"),
            ("2024-02-05", "Runebergsdagen"),
            ("2024-02-28", "Kalevaladagen, den finska kulturens dag"),
            ("2024-03-19", "Minna Canth-dagen, jämställdhetsdagen"),
            ("2024-03-29", "Långfredagen"),
            ("2024-03-31", "Påskdagen"),
            ("2024-04-01", "Annandag påsk"),
            ("2024-04-09", "Mikael Agricoladagen, finska språkets dag"),
            ("2024-04-27", "Nationella veterandagen"),
            ("2024-05-01", "Första maj"),
            ("2024-05-09", "Europadagen; Kristi himmelsfärdsdag"),
            ("2024-05-12", "Mors dag; Snellmansdagen, finskhetens dag"),
            ("2024-05-19", "De stupades dag; Pingst"),
            ("2024-06-06", "Dagen för försvarets fanfest"),
            ("2024-06-09", "Ålands självstyrelsedag"),
            ("2024-06-21", "Midsommarafton"),
            ("2024-06-22", "Finlands flaggas dag; Midsommardagen"),
            ("2024-07-06", "Eino Leino-dagen, diktens och sommarens dag"),
            ("2024-08-31", "Den finska naturens dag"),
            ("2024-10-01", "Miina Sillanpää-dagen, medborgarinflytandets dag"),
            ("2024-10-10", "Aleksis Kivi-dagen, den finska litteraturens dag"),
            ("2024-10-24", "FN-dagen"),
            ("2024-11-02", "Alla helgons dag"),
            ("2024-11-06", "Svenska dagen, Gustav Adolfsdagen"),
            ("2024-11-10", "Fars dag"),
            ("2024-11-20", "Barnkonventionens dag"),
            ("2024-12-06", "Självständighetsdagen"),
            ("2024-12-08", "Sibeliusdagen, den finländska musikens dag"),
            ("2024-12-24", "Julafton"),
            ("2024-12-25", "Juldagen"),
            ("2024-12-26", "Annandag jul"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2024-01-01", "วันขึ้นปีใหม่"),
            ("2024-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2024-02-05", "วันรูนแบร์ก"),
            ("2024-02-28", "วันกาเลวาลา, วันวัฒนธรรมฟินแลนด์"),
            ("2024-03-19", "วันมินน่า คานท์, วันแห่งความเสมอภาค"),
            ("2024-03-29", "วันศุกร์ประเสริฐ"),
            ("2024-03-31", "วันอาทิตย์อีสเตอร์"),
            ("2024-04-01", "วันจันทร์อีสเตอร์"),
            ("2024-04-09", "วันมิคาเอล อากริโคลา, วันภาษาฟินแลนด์"),
            ("2024-04-27", "วันทหารผ่านศึกแห่งชาติ"),
            ("2024-05-01", "วันเมย์เดย์ (วันแรงงาน)"),
            ("2024-05-09", "วันยุโรป; วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2024-05-12", "วันเจ.วี. สเนลล์มาน, วันมรดกฟินแลนด์; วันแม่"),
            ("2024-05-19", "วันรำลึก; วันสมโภชพระจิตเจ้า"),
            ("2024-06-06", "วันกองกำลังป้องกันฟินแลนด์"),
            ("2024-06-09", "วันปกครองตนเองหมู่เกาะโอลันด์"),
            ("2024-06-21", "วันมิดซัมเมอร์อีฟ (วันก่อนวันกลางฤดูร้อน)"),
            ("2024-06-22", "วันธงชาติฟินแลนด์; วันมิดซัมเมอร์ (วันกลางฤดูร้อน)"),
            ("2024-07-06", "วันไอนอ ไลโน, วันแห่งบทกวีและฤดูร้อน"),
            ("2024-08-31", "วันสิ่งแวดล้อมฟินแลนด์"),
            ("2024-10-01", "วันมีนา ซิลลันแป, วันส่งเสริมการมีส่วนร่วมของพลเมือง"),
            ("2024-10-10", "วันอเล็กซิส กีวี, วันวรรณกรรมฟินแลนด์"),
            ("2024-10-24", "วันสหประชาชาติ"),
            ("2024-11-02", "วันสมโภชนักบุญทั้งหลาย"),
            ("2024-11-06", "วันมรดกสวีเดน-ฟินแลนด์, วันกุสตาฟวัส อดอลฟัส"),
            ("2024-11-10", "วันพ่อ"),
            ("2024-11-20", "วันสิทธิเด็ก"),
            ("2024-12-06", "วันประกาศอิสรภาพฟินแลนด์"),
            ("2024-12-08", "วันฌอง ซิเบลิอุส, วันดนตรีฟินแลนด์"),
            ("2024-12-24", "วันคริสต์มาสอีฟ"),
            ("2024-12-25", "วันคริสต์มาส"),
            ("2024-12-26", "วันคริสต์มาสวันที่สอง"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-01-06", "Богоявлення"),
            ("2024-02-05", "День Рунеберга"),
            ("2024-02-28", "День Калевали, День фінської культури"),
            ("2024-03-19", "День Мінни Кант, День рівності"),
            ("2024-03-29", "Страсна пʼятниця"),
            ("2024-03-31", "Великдень"),
            ("2024-04-01", "Великодній понеділок"),
            ("2024-04-09", "День Мікаеля Аґріколи, День фінської мови"),
            ("2024-04-27", "Національний день ветеранів"),
            ("2024-05-01", "Ваппу"),
            ("2024-05-09", "Вознесіння Господнє; День Європи"),
            ("2024-05-12", "День Ю. В. Снелльмана, День фінської спадщини; День матері"),
            ("2024-05-19", "День ветеранів Національної війни; Трійця"),
            ("2024-06-06", "День прапора фінських сил оборони"),
            ("2024-06-09", "День автономії Аландських островів"),
            ("2024-06-21", "Переддень літнього сонцестояння"),
            ("2024-06-22", "День літнього сонцестояння; День прапора Фінляндії"),
            ("2024-07-06", "День Ейно Лейно, День літа та поезії"),
            ("2024-08-31", "День природи Фінляндії"),
            ("2024-10-01", "День Міїни Сілланпяя, День громадянської активності"),
            ("2024-10-10", "День Алексіса Ківі, День фінської літератури"),
            ("2024-10-24", "День ООН"),
            ("2024-11-02", "День усіх святих"),
            ("2024-11-06", "День фінської шведської спадщини, шведський день"),
            ("2024-11-10", "День батька"),
            ("2024-11-20", "День прав дитини"),
            ("2024-12-06", "День незалежності"),
            ("2024-12-08", "День Жана Сібеліуса, День фінської музики"),
            ("2024-12-24", "Святий вечір"),
            ("2024-12-25", "Різдво Христове"),
            ("2024-12-26", "Другий день Різдва"),
        )



#!/usr/bin/env python3


#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

"""Generate Gregorian dates for holidays based on the Chinese lunisolar calendar.

Run with:

    python -m scripts.calendar.asian_generator

Alternatively, run with uv:

    uv run -m scripts.calendar.asian_generator

This generates the files:

    * `holidays/calendars/buddhist_dates.py`
    * `holidays/calendars/chinese_dates.py`
    * `holidays/calendars/hindu_dates.py`

whose data can then be copied to:

    * `holidays/calendars/buddhist.py`
    * `holidays/calendars/chinese.py`
    * `holidays/calendars/hindu.py`
"""

from collections import defaultdict
from datetime import date
from datetime import timedelta as td
from functools import cache

from .generator import CalendarGenerator


class _Lunisolar:
    """Convert dates from the Chinese lunisolar calendar to Gregorian dates.

    References:
        * <https://en.wikipedia.org/wiki/Chinese_calendar>
        * <https://github.com/BohanZhang/python-MagicLamp/blob/master/bin/utils/lunar_calendar.py>
    """

    def __init__(self) -> None:

        # Bit-encoded lunar calendar data starting from year 1901.
        # Bits 1-13 encode month lengths for the 12 regular months and
        # the optional leap month: 1 = 30 days, 0 = 29 days.
        # Bits 16-19 encode the leap month number.
        # A value of 0x0F means that the year has no leap month.
        self.G_LUNAR_MONTH_DAYS = (
            0xF0EA4,
            0xF1D4A,
            0x52C94,
            0xF0C96,
            0xF1536,
            0x42AAC,
            0xF0AD4,
            0xF16B2,
            0x22EA4,
            0xF0EA4,  # 1910.
            0x6364A,
            0xF164A,
            0xF1496,
            0x52556,
            0xF155A,
            0xF0AD4,
            0x216D2,
            0xF1B52,
            0x73B24,
            0xF1B24,  # 1920.
            0xF1A4A,
            0x5349A,
            0xF14AC,
            0xF056C,
            0x42B6A,
            0xF0DA8,
            0xF1D52,
            0x23D24,
            0xF1D24,
            0x61A4C,  # 1930.
            0xF0A56,
            0xF14AE,
            0x5256C,
            0xF16B4,
            0xF0DA8,
            0x31D92,
            0xF0E92,
            0x72D26,
            0xF1526,
            0xF0A56,  # 1940.
            0x614B6,
            0xF155A,
            0xF0AD4,
            0x436AA,
            0xF1748,
            0xF1692,
            0x23526,
            0xF152A,
            0x72A5A,
            0xF0A6C,  # 1950.
            0xF155A,
            0x52B54,
            0xF0B64,
            0xF1B4A,
            0x33A94,
            0xF1A94,
            0x8152A,
            0xF152E,
            0xF0AAC,
            0x6156A,  # 1960.
            0xF15AA,
            0xF0DA4,
            0x41D4A,
            0xF1D4A,
            0xF0C94,
            0x3192E,
            0xF1536,
            0x72AB4,
            0xF0AD4,
            0xF16D2,  # 1970.
            0x52EA4,
            0xF16A4,
            0xF164A,
            0x42C96,
            0xF1496,
            0x82956,
            0xF055A,
            0xF0ADA,
            0x616D2,
            0xF1B52,  # 1980.
            0xF1B24,
            0x43A4A,
            0xF1A4A,
            0xA349A,
            0xF14AC,
            0xF056C,
            0x60B6A,
            0xF0DAA,
            0xF1D52,
            0x53D24,  # 1990.
            0xF1D24,
            0xF1A4C,
            0x314AC,
            0xF14AE,
            0x829AC,
            0xF06B4,
            0xF0DAA,
            0x52D92,
            0xF0E92,
            0xF0D26,  # 2000.
            0x42A56,
            0xF0A56,
            0xF14B6,
            0x22AB4,
            0xF0AD4,
            0x736AA,
            0xF1748,
            0xF1692,
            0x53526,
            0xF152A,  # 2010.
            0xF0A5A,
            0x4155A,
            0xF156A,
            0x92B54,
            0xF0BA4,
            0xF1B4A,
            0x63A94,
            0xF1A94,
            0xF192A,
            0x42A5C,  # 2020.
            0xF0AAC,
            0xF156A,
            0x22B64,
            0xF0DA4,
            0x61D4A,
            0xF0E4A,
            0xF0C96,
            0x5192E,
            0xF1956,
            0xF0AB4,  # 2030.
            0x315AC,
            0xF16D2,
            0xB2EA4,
            0xF16A4,
            0xF164A,
            0x63496,
            0xF1496,
            0xF0956,
            0x50AB6,
            0xF0B5A,  # 2040.
            0xF16D4,
            0x236A4,
            0xF1B24,
            0x73A4A,
            0xF1A4A,
            0xF14AA,
            0x5295A,
            0xF096C,
            0xF0B6A,
            0x31B54,  # 2050.
            0xF1D92,
            0x83D24,
            0xF1D24,
            0xF1A4C,
            0x614AC,
            0xF14AE,
            0xF09AC,
            0x40DAA,
            0xF0EAA,
            0xF0E92,  # 2060.
            0x31D26,
            0xF0D26,
            0x72A56,
            0xF0A56,
            0xF14B6,
            0x52AB4,
            0xF0AD4,
            0xF16CA,
            0x42E94,
            0xF1694,  # 2070.
            0x8352A,
            0xF152A,
            0xF0A5A,
            0x6155A,
            0xF156A,
            0xF0B54,
            0x4174A,
            0xF1B4A,
            0xF1A94,
            0x3392A,  # 2080.
            0xF192C,
            0x7329C,
            0xF0AAC,
            0xF156A,
            0x52B64,
            0xF0DA4,
            0xF1D4A,
            0x41C94,
            0xF0D16,
            0x8192E,  # 2090.
            0xF0956,
            0xF0AB6,
            0x615AC,
            0xF16D4,
            0xF0EA4,
            0x42E4A,
            0xF164A,
            0xF1516,
            0x22936,
            0xF0956,  # 2100.
        )
        # Define range of years covered.
        self.START_YEAR = 1901
        self.END_YEAR = 2100
        # The Gregorian date for the 1st day of the 1st month of 1900 year is 1901-02-19.
        self.LUNAR_START_DATE = ((1901, 1, 1),)
        self.SOLAR_START_DATE = date(1901, 2, 19)
        # The Gregorian date for the 29th day of 12th month of 2100 year is 2101-01-28.
        self.LUNAR_END_DATE = (2100, 12, 29)
        self.SOLAR_END_DATE = date(2101, 1, 28)

    @cache
    def _get_leap_month(self, lunar_year: int) -> int:
        """Return the number of the leap month if one exists in the year, otherwise 15."""
        return (self.G_LUNAR_MONTH_DAYS[lunar_year - self.START_YEAR] >> 16) & 0x0F

    def _lunar_month_days(self, lunar_year: int, lunar_month: int) -> int:
        """Return the number of days in a lunar month."""
        return 29 + ((self.G_LUNAR_MONTH_DAYS[lunar_year - self.START_YEAR] >> lunar_month) & 0x01)

    def _lunar_year_days(self, year: int) -> int:
        """Return the number of days in a lunar year."""
        days = 0
        months_day = self.G_LUNAR_MONTH_DAYS[year - self.START_YEAR]
        for i in range(1, 13 if self._get_leap_month(year) == 0x0F else 14):
            day = 29 + ((months_day >> i) & 0x01)
            days += day
        return days

    @cache
    def _span_days(self, year: int) -> int:
        """Return elapsed days from SOLAR_START_DATE to start of year."""
        return sum(self._lunar_year_days(y) for y in range(self.START_YEAR, year))

    def lunar_to_gre(self, year: int, month: int, day: int, *, leap: bool = True) -> date:
        """Return the Gregorian date corresponding to a Chinese lunar
        month and day in a given Gregorian year."""
        leap_month = self._get_leap_month(year) if leap else 15
        span_days = sum(
            self._lunar_month_days(year, m) for m in range(1, month + (month > leap_month))
        )
        span_days += self._span_days(year) + day - 1
        return self.SOLAR_START_DATE + td(days=span_days)

    def vesak_may_date(self, year: int) -> date:
        """Return the estimated Gregorian date of Vesak for Sri Lanka, Nepal,
        India, Bangladesh and Malaysia, corresponding to the day of the
        first full moon in May in the Gregorian calendar.

        See <https://en.wikipedia.org/wiki/Vesak#Dates_of_observance>.
        """
        dt = self.lunar_to_gre(year, 3, 15, leap=False)
        return dt if dt.month == 5 else self.lunar_to_gre(year, 4, 15, leap=False)

    def thaipusam_date(self, year: int) -> date:
        """Return the estimated Gregorian date of Thaipusam (Tamil). Defined as
        the date of the full moon in the Tamil month of Thai, which
        corresponds with the months of January or February in the Gregorian
        calendar.

        See <https://en.wikipedia.org/wiki/Thaipusam>.
        """
        if year == self.START_YEAR:
            return self.lunar_to_gre(year, 1, 15) - td(days=30)

        y = year - 1
        m = 12
        if self._get_leap_month(year) < 15:
            y += 1
            m = 1
        return self.lunar_to_gre(y, m, 15)


BUDDHIST = "buddhist"
CHINESE = "chinese"
HINDU = "hindu"

# Timezones:
# UTC+7: VN,
# UTC+8: BN, CN, HK, ID, MO, MY, PH, SG, TW,
# UTC+9: KR.

ASIAN_HOLIDAYS: dict[str, dict[str, tuple[int, int] | None]] = {
    CHINESE: {
        # CN, HK, ID, KR, MY, PH, SG, TW, VN.
        "LUNAR_NEW_YEAR": (1, 1),  # Lunar New Year.
        # VN.
        "HUNG_KINGS": (3, 10),  # Kings' Commemoration Day.
        # HK, KR.
        "BUDDHA_BIRTHDAY": (4, 8),  # Buddha's Birthday.
        # CN, HK, TW.
        "DRAGON_BOAT": (5, 5),  # Dragon Boat Festival.
        # CN, HK, KR, TW.
        "MID_AUTUMN": (8, 15),  # Mid-Autumn Festival.
        # HK.
        "DOUBLE_NINTH": (9, 9),  # Double Ninth Festival.
    },
    BUDDHIST: {
        # ID, SG.
        "VESAK": (4, 15),  # Vesak.
        # MY.
        "VESAK_MAY": None,  # Vesak (May).
    },
    HINDU: {
        # KE, LK, MY, SG, SR, TT.
        "DIWALI": (9, -1),  # Diwali.
        # MU, MY.
        "THAIPUSAM": None,  # Thaipusam.
    },
}


def generate_data() -> None:
    cal = _Lunisolar()
    years = range(1901, 2101)

    for calendar in (BUDDHIST, CHINESE, HINDU):
        dates: dict[str, dict[int, date]] = defaultdict(dict)
        for name, data in ASIAN_HOLIDAYS[calendar].items():
            if data:
                holiday_month, holiday_day = data
                for year in years:
                    dates[name][year] = cal.lunar_to_gre(
                        year,
                        holiday_month,
                        holiday_day
                        if holiday_day > 0
                        else cal._lunar_month_days(
                            year, holiday_month + (holiday_month > cal._get_leap_month(year))
                        )
                        + holiday_day
                        + 1,
                    )
            else:
                holiday_func = getattr(cal, f"{name.lower()}_date")
                for year in years:
                    dates[name][year] = holiday_func(year)

        cal_gen = CalendarGenerator(calendar, f"_{calendar.capitalize()}Lunisolar")
        cal_gen.generate(dates)


if __name__ == "__main__":
    generate_data()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.utils import list_localized_countries, list_localized_financial
from scripts.generate_site_assets import LANGUAGES


class TestGenerateSiteAssets(TestCase):
    def test_languages_mapping(self):
        languages = set(LANGUAGES)
        supported_languages = set().union(
            *list_localized_countries(include_aliases=False).values(),
            *list_localized_financial(include_aliases=False).values(),
        )
        self.assertEqual(
            languages,
            supported_languages,
            "LANGUAGES in `generate_site_assets.py` does not match supported languages list; "
            f"missing: {supported_languages - languages or 'no'}, "
            f"extra: {languages - supported_languages or 'no'}",
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import MAY, SUN, _timedelta, _get_nth_weekday_of_month
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Madagascar(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Madagascar holidays.

    References:
        * <https://web.archive.org/web/20250114061412/https://www.officeholidays.com/countries/madagascar>
        * <https://web.archive.org/web/20250413112126/https://www.timeanddate.com/holidays/madagascar/>
    """

    country = "MG"
    default_language = "mg"
    supported_languages = ("en_US", "mg", "uk")
    start_year = 1947

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Taom-baovao"))

        # Women's Day.
        self._add_womens_day(tr("Fetin'ny vehivavy"))

        # Martyrs' Day.
        self._add_holiday_mar_29(tr("Fetin'ny mahery fo"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Fetin'ny paska"))

        # Easter Monday.
        self._add_easter_monday(tr("Alatsinain'ny paska"))

        # Labor Day.
        self._add_labor_day(tr("Fetin'ny asa"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Fiakaran'ny Jesosy kristy tany an-danitra"))

        # Whit Sunday.
        whit_sunday = self._add_whit_sunday(tr("Pentekosta"))

        # Whit Monday.
        self._add_whit_monday(tr("Alatsinain'ny pentekosta"))

        last_sun_of_may = _get_nth_weekday_of_month(-1, SUN, MAY, self._year)
        self._add_holiday(
            # Mother's Day.
            tr("Fetin'ny reny"),
            _timedelta(last_sun_of_may, +7) if last_sun_of_may == whit_sunday else last_sun_of_may,
        )

        # Father's Day.
        self._add_holiday_3rd_sun_of_jun(tr("Fetin'ny ray"))

        if self._year >= 1960:
            # Independence Day.
            self._add_holiday_jun_26(tr("Fetin'ny fahaleovantena"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Fiakaran'ny Masina Maria tany an-danitra"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Fetin'ny olo-masina"))

        if self._year >= 2011:
            # Republic Day.
            self._add_holiday_dec_11(tr("Fetin'ny Repoblika"))

        # Christmas Day.
        self._add_christmas_day(tr("Fetin'ny noely"))


class MG(Madagascar):
    pass


class MDG(Madagascar):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.isle_of_man import IsleOfMan
from tests.common import CommonCountryTests


class TestIsleOfMan(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(IsleOfMan)

    def test_1970(self):
        self.assertHolidaysInYear(
            1970,
            ("1970-03-27", "Good Friday"),
            ("1970-03-30", "Easter Monday"),
            ("1970-05-18", "Whit Monday"),
            ("1970-06-05", "TT Bank Holiday"),
            ("1970-07-05", "Tynwald Day"),
            ("1970-12-25", "Christmas Day"),
            ("1970-12-26", "Boxing Day"),
            ("1970-12-28", "Boxing Day (observed)"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-02", "May Day"),
            ("2022-06-02", "Spring Bank Holiday"),
            ("2022-06-03", "Platinum Jubilee of Elizabeth II; TT Bank Holiday"),
            ("2022-07-05", "Tynwald Day"),
            ("2022-08-29", "Late Summer Bank Holiday"),
            ("2022-09-19", "State Funeral of Queen Elizabeth II"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_tt_bank_holiday(self):
        name = "TT Bank Holiday"
        self.assertHolidayName(
            name,
            "2020-06-05",
            "2021-06-04",
            "2022-06-03",
            "2023-06-02",
            "2024-06-07",
            "2025-06-06",
        )
        self.assertHolidayName(name, self.full_range)

    def test_tynwald_day(self):
        name = "Tynwald Day"
        self.assertNonObservedHolidayName(name, (f"{year}-07-05" for year in self.full_range))
        obs_dts = (
            "1992-07-06",
            "1997-07-07",
            "1998-07-06",
            "2003-07-07",
            "2008-07-07",
            "2009-07-06",
            "2014-07-07",
            "2015-07-06",
            "2020-07-06",
            "2025-07-07",
        )
        self.assertHolidayName(name, obs_dts)
        self.assertNoNonObservedHolidayName(name, obs_dts)

    def test_l10n_default(self):
        # https://www.gov.im/categories/home-and-neighbourhood/bank-holidays/
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "May Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-06-07", "TT Bank Holiday"),
            ("2024-07-05", "Tynwald Day"),
            ("2024-08-26", "Late Summer Bank Holiday"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "May Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-06-07", "TT Bank Holiday"),
            ("2024-07-05", "Tynwald Day"),
            ("2024-08-26", "Late Summer Bank Holiday"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2024-01-01", "วันขึ้นปีใหม่"),
            ("2024-03-29", "วันศุกร์ประเสริฐ"),
            ("2024-04-01", "วันจันทร์อีสเตอร์"),
            ("2024-05-06", "วันเมย์เดย์"),
            ("2024-05-27", "วันหยุดฤดูใบไม้ผลิของธนาคาร"),
            ("2024-06-07", "วันแข่งไอร์ออฟแมน ทีที"),
            ("2024-07-05", "วันไทน์วอลด์"),
            ("2024-08-26", "วันหยุดช่วงปลายฤดูร้อนของธนาคาร"),
            ("2024-12-25", "วันคริสต์มาส"),
            ("2024-12-26", "วันเปิดกล่องของขวัญ"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.kazakhstan import Kazakhstan
from tests.common import CommonCountryTests, WorkingDayTests


class TestKazakhstan(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Kazakhstan)

    def test_substituted_holidays(self):
        self.assertHoliday(
            "2000-05-08",
            "2001-03-09",
            "2001-03-23",
            "2001-04-30",
            "2001-12-31",
            "2002-05-10",
            "2003-05-02",
            "2003-12-15",
            "2005-03-07",
            "2005-03-21",
            "2005-08-29",
            "2005-10-24",
            "2006-01-11",
            "2006-05-08",
            "2007-03-09",
            "2007-03-23",
            "2007-08-31",
            "2007-10-26",
            "2007-12-31",
            "2008-05-02",
            "2009-12-18",
            "2010-01-08",
            "2010-07-05",
            "2011-03-07",
            "2011-08-29",
            "2012-03-09",
            "2012-04-30",
            "2012-12-31",
            "2013-05-10",
            "2013-10-14",
            "2014-01-03",
            "2014-05-02",
            "2014-05-08",
            "2016-03-07",
            "2017-03-20",
            "2017-07-07",
            "2018-03-09",
            "2018-04-30",
            "2018-05-08",
            "2018-08-31",
            "2018-12-31",
            "2019-05-10",
            "2020-01-03",
            "2020-12-18",
            "2021-07-05",
            "2022-03-07",
            "2022-08-29",
            "2022-10-24",
            "2023-07-07",
            "2024-05-08",
            "2025-01-03",
        )

    def test_workdays(self):
        self.assertWorkingDay(
            "2000-05-06",
            "2001-03-11",
            "2001-03-25",
            "2001-04-28",
            "2001-12-29",
            "2002-05-12",
            "2003-05-04",
            "2003-12-13",
            "2005-03-05",
            "2005-03-19",
            "2005-08-27",
            "2005-10-22",
            "2006-01-14",
            "2006-05-06",
            "2007-03-11",
            "2007-03-25",
            "2007-09-02",
            "2007-10-28",
            "2007-12-29",
            "2008-05-04",
            "2009-12-20",
            "2010-01-10",
            "2010-07-03",
            "2011-03-05",
            "2011-08-27",
            "2012-03-11",
            "2012-04-28",
            "2012-12-29",
            "2013-05-04",
            "2013-10-12",
            "2013-12-28",
            "2014-05-04",
            "2014-05-11",
            "2016-03-05",
            "2017-03-18",
            "2017-07-01",
            "2018-03-03",
            "2018-04-28",
            "2018-05-05",
            "2018-08-25",
            "2018-12-29",
            "2019-05-04",
            "2020-01-05",
            "2020-12-20",
            "2021-07-03",
            "2022-03-05",
            "2022-08-27",
            "2022-10-22",
            "2023-07-01",
            "2024-05-04",
            "2025-01-05",
        )

        for year, dts in {
            2013: (
                "2013-05-04",
                "2013-10-12",
                "2013-12-28",
            ),
        }.items():
            self.assertWorkingDay(Kazakhstan(years=year), dts)

    def test_new_years_day(self):
        name = "Жаңа жыл"
        self.assertHolidayName(
            name,
            (f"{year}-01-01" for year in self.full_range),
            (f"{year}-01-02" for year in self.full_range),
        )
        obs_dts = (
            "2017-01-03",
            "2021-01-04",
            "2022-01-03",
            "2022-01-04",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_orthodox_christmas(self):
        name = "Православиелік Рождество"
        self.assertHolidayName(name, (f"{year}-01-07" for year in range(2006, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2006))

    def test_international_womens_day(self):
        name = "Халықаралық әйелдер күні"
        self.assertHolidayName(name, (f"{year}-03-08" for year in self.full_range))
        obs_dts = (
            "2009-03-09",
            "2014-03-10",
            "2015-03-09",
            "2020-03-09",
            "2025-03-10",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_nowruz_holiday(self):
        name = "Наурыз мейрамы"
        self.assertHolidayName(
            name,
            (f"{year}-03-21" for year in range(2010, self.end_year)),
            (f"{year}-03-22" for year in range(2002, self.end_year)),
            (f"{year}-03-23" for year in range(2010, self.end_year)),
        )
        self.assertHolidayNameCount(name, 1, range(2002, 2010))
        self.assertHolidayNameCount(name, 3, range(2010, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2002))
        obs_dts = (
            "2009-03-23",
            "2021-03-24",
            "2024-03-25",
            "2025-03-24",
            "2025-03-25",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_kazakhstans_people_solidarity_holiday(self):
        name = "Қазақстан халқының бірлігі мерекесі"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_defender_of_the_fatherland_day(self):
        name = "Отан Қорғаушы күні"
        self.assertHolidayName(name, (f"{year}-05-07" for year in range(2013, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2013))
        obs_dts = (
            "2016-05-10",
            "2017-05-08",
            "2022-05-10",
            "2023-05-08",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_victory_day(self):
        name = "Жеңіс күні"
        self.assertHolidayName(name, (f"{year}-05-09" for year in self.full_range))
        obs_dts = (
            "2009-05-11",
            "2010-05-10",
            "2015-05-11",
            "2021-05-10",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_capital_day(self):
        name = "Астана күні"
        self.assertHolidayName(name, (f"{year}-07-06" for year in range(2009, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2009))
        obs_dts = (
            "2013-07-08",
            "2014-07-07",
            "2019-07-08",
            "2024-07-08",
            "2025-07-07",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_constitution_day(self):
        name = "Қазақстан Республикасының Конституциясы күні"
        self.assertHolidayName(name, (f"{year}-08-30" for year in range(1996, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1996))
        obs_dts = (
            "2009-08-31",
            "2014-09-01",
            "2015-08-31",
            "2020-08-31",
            "2025-09-01",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_republic_day(self):
        name = "Республика күні"
        self.assertHolidayName(
            name, (f"{year}-10-25" for year in (*range(1994, 2009), *range(2022, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1994), range(2009, 2022))
        obs_dts = (
            "2003-10-27",
            "2008-10-27",
            "2025-10-27",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_first_president_day(self):
        name = "Қазақстан Республикасының Тұңғыш Президенті күні"
        self.assertHolidayName(name, (f"{year}-12-01" for year in range(2012, 2022)))
        self.assertNoHolidayName(name, range(self.start_year, 2012), range(2022, self.end_year))
        obs_dts = (
            "2012-12-03",
            "2013-12-02",
            "2018-12-03",
            "2019-12-02",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Тəуелсіздік күні"
        self.assertHolidayName(name, (f"{year}-12-16" for year in self.full_range))
        self.assertHolidayName(name, (f"{year}-12-17" for year in range(2002, 2022)))
        self.assertHolidayNameCount(
            name, 1, range(self.start_year, 2002), range(2022, self.end_year)
        )
        obs_dts = (
            "2016-12-19",
            "2017-12-18",
            "2017-12-19",
            "2018-12-18",
            "2023-12-18",
        )
        self.assertHolidayName(f"{name} (қайта белгіленген демалыс)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Құрбан айт"
        self.assertHolidayName(
            name,
            "2006-01-10",
            "2007-12-20",
            "2008-12-08",
            "2009-11-27",
            "2010-11-16",
            "2011-11-06",
            "2012-10-26",
            "2013-10-15",
            "2014-10-04",
            "2015-09-24",
            "2016-09-12",
            "2017-09-01",
            "2018-08-21",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(2006, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 2006))

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Жаңа жыл"),
            ("2022-01-02", "Жаңа жыл"),
            ("2022-01-03", "Жаңа жыл (қайта белгіленген демалыс)"),
            ("2022-01-04", "Жаңа жыл (қайта белгіленген демалыс)"),
            ("2022-01-07", "Православиелік Рождество"),
            ("2022-03-07", "Демалыс күні (05.03.2022 бастап ауыстырылды)"),
            ("2022-03-08", "Халықаралық әйелдер күні"),
            ("2022-03-21", "Наурыз мейрамы"),
            ("2022-03-22", "Наурыз мейрамы"),
            ("2022-03-23", "Наурыз мейрамы"),
            ("2022-05-01", "Қазақстан халқының бірлігі мерекесі"),
            ("2022-05-02", "Қазақстан халқының бірлігі мерекесі (қайта белгіленген демалыс)"),
            ("2022-05-07", "Отан Қорғаушы күні"),
            ("2022-05-09", "Жеңіс күні"),
            ("2022-05-10", "Отан Қорғаушы күні (қайта белгіленген демалыс)"),
            ("2022-07-06", "Астана күні"),
            ("2022-07-09", "Құрбан айт"),
            ("2022-08-29", "Демалыс күні (27.08.2022 бастап ауыстырылды)"),
            ("2022-08-30", "Қазақстан Республикасының Конституциясы күні"),
            ("2022-10-24", "Демалыс күні (22.10.2022 бастап ауыстырылды)"),
            ("2022-10-25", "Республика күні"),
            ("2022-12-16", "Тəуелсіздік күні"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Жаңа жыл"),
            ("2024-01-02", "Жаңа жыл"),
            ("2024-01-07", "Православиелік Рождество"),
            ("2024-03-08", "Халықаралық әйелдер күні"),
            ("2024-03-21", "Наурыз мейрамы"),
            ("2024-03-22", "Наурыз мейрамы"),
            ("2024-03-23", "Наурыз мейрамы"),
            ("2024-03-25", "Наурыз мейрамы (қайта белгіленген демалыс)"),
            ("2024-05-01", "Қазақстан халқының бірлігі мерекесі"),
            ("2024-05-07", "Отан Қорғаушы күні"),
            ("2024-05-08", "Демалыс күні (04.05.2024 бастап ауыстырылды)"),
            ("2024-05-09", "Жеңіс күні"),
            ("2024-06-16", "Құрбан айт"),
            ("2024-07-06", "Астана күні"),
            ("2024-07-08", "Астана күні (қайта белгіленген демалыс)"),
            ("2024-08-30", "Қазақстан Республикасының Конституциясы күні"),
            ("2024-10-25", "Республика күні"),
            ("2024-12-16", "Тəуелсіздік күні"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "New Year's Day"),
            ("2024-01-07", "Orthodox Christmas"),
            ("2024-03-08", "International Women's Day"),
            ("2024-03-21", "Nowruz Holiday"),
            ("2024-03-22", "Nowruz Holiday"),
            ("2024-03-23", "Nowruz Holiday"),
            ("2024-03-25", "Nowruz Holiday (observed)"),
            ("2024-05-01", "Kazakhstan's People Solidarity Holiday"),
            ("2024-05-07", "Defender of the Fatherland Day"),
            ("2024-05-08", "Day off (substituted from 05/04/2024)"),
            ("2024-05-09", "Victory Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-07-06", "Capital Day"),
            ("2024-07-08", "Capital Day (observed)"),
            ("2024-08-30", "Constitution Day"),
            ("2024-10-25", "Republic Day"),
            ("2024-12-16", "Independence Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-01-02", "Новий рік"),
            ("2024-01-07", "Православне Різдво"),
            ("2024-03-08", "Міжнародний жіночий день"),
            ("2024-03-21", "Свято Новруз"),
            ("2024-03-22", "Свято Новруз"),
            ("2024-03-23", "Свято Новруз"),
            ("2024-03-25", "Свято Новруз (вихідний)"),
            ("2024-05-01", "Свято єдності народу Казахстану"),
            ("2024-05-07", "День захисника Вітчизни"),
            ("2024-05-08", "Вихідний день (перенесено з 04.05.2024)"),
            ("2024-05-09", "День Перемоги"),
            ("2024-06-16", "Курбан-байрам"),
            ("2024-07-06", "День Столиці"),
            ("2024-07-08", "День Столиці (вихідний)"),
            ("2024-08-30", "День Конституції Республіки Казахстан"),
            ("2024-10-25", "День Республіки"),
            ("2024-12-16", "День незалежності"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cyprus import Cyprus
from tests.common import CommonCountryTests


class TestCyprus(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Cyprus)

    def test_new_years_day(self):
        self.assertHolidayName("Πρωτοχρονιά", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        self.assertHolidayName(
            "Ημέρα των Θεοφανίων", (f"{year}-01-06" for year in self.full_range)
        )

    def test_green_monday(self):
        name = "Καθαρά Δευτέρα"
        self.assertHolidayName(
            name,
            "2020-03-02",
            "2021-03-15",
            "2022-03-07",
            "2023-02-27",
            "2024-03-18",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_greek_independence_day(self):
        self.assertHolidayName(
            "Ημέρα της Ελληνικής Ανεξαρτησίας", (f"{year}-03-25" for year in self.full_range)
        )

    def test_cyprus_national_day(self):
        self.assertHolidayName(
            "Εθνική Ημέρα Κύπρου", (f"{year}-04-01" for year in self.full_range)
        )

    def test_good_friday(self):
        name = "Μεγάλη Παρασκευή"
        self.assertHolidayName(
            name,
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Μεγάλο Σάββατο"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-04-18",
            "2021-05-01",
            "2022-04-23",
            "2023-04-15",
            "2024-05-04",
            "2025-04-19",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Κυριακή του Πάσχα"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Δευτέρα της Διακαινησίμου"
        self.assertHolidayName(
            name,
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_tuesday(self):
        name = "Τρίτη της Διακαινησίμου"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(
            name,
            "2020-04-21",
            "2021-05-04",
            "2022-04-26",
            "2023-04-18",
            "2024-05-07",
            "2025-04-22",
        )
        self.assertBankHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Πρωτομαγιά", (f"{year}-05-01" for year in self.full_range))

    def test_whit_monday(self):
        name = "Δευτέρα του Αγίου Πνεύματος"
        self.assertHolidayName(
            name,
            "2020-06-08",
            "2021-06-21",
            "2022-06-13",
            "2023-06-05",
            "2024-06-24",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        self.assertHolidayName(
            "Κοίμηση της Θεοτόκου", (f"{year}-08-15" for year in self.full_range)
        )

    def test_cyprus_independence_day(self):
        name = "Ημέρα της Κυπριακής Ανεξαρτησίας"
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(1979, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1979))

    def test_ochi_day(self):
        self.assertHolidayName("Ημέρα του Όχι", (f"{year}-10-28" for year in self.full_range))

    def test_christmas_eve(self):
        name = "Παραμονή Χριστουγέννων"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("Χριστούγεννα", (f"{year}-12-25" for year in self.full_range))

    def test_day_after_christmas(self):
        self.assertHolidayName(
            "Επομένη Χριστουγέννων", (f"{year}-12-26" for year in self.full_range)
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Πρωτοχρονιά"),
            ("2023-01-06", "Ημέρα των Θεοφανίων"),
            ("2023-02-27", "Καθαρά Δευτέρα"),
            ("2023-03-25", "Ημέρα της Ελληνικής Ανεξαρτησίας"),
            ("2023-04-01", "Εθνική Ημέρα Κύπρου"),
            ("2023-04-14", "Μεγάλη Παρασκευή"),
            ("2023-04-15", "Μεγάλο Σάββατο"),
            ("2023-04-16", "Κυριακή του Πάσχα"),
            ("2023-04-17", "Δευτέρα της Διακαινησίμου"),
            ("2023-04-18", "Τρίτη της Διακαινησίμου"),
            ("2023-05-01", "Πρωτομαγιά"),
            ("2023-06-05", "Δευτέρα του Αγίου Πνεύματος"),
            ("2023-08-15", "Κοίμηση της Θεοτόκου"),
            ("2023-10-01", "Ημέρα της Κυπριακής Ανεξαρτησίας"),
            ("2023-10-28", "Ημέρα του Όχι"),
            ("2023-12-24", "Παραμονή Χριστουγέννων"),
            ("2023-12-25", "Χριστούγεννα"),
            ("2023-12-26", "Επομένη Χριστουγέννων"),
        )

    def test_l10n_en_cy(self):
        self.assertLocalizedHolidays(
            "en_CY",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-06", "Epiphany"),
            ("2023-02-27", "Green Monday"),
            ("2023-03-25", "Greek Independence Day"),
            ("2023-04-01", "Cyprus National Day"),
            ("2023-04-14", "Good Friday"),
            ("2023-04-15", "Holy Saturday"),
            ("2023-04-16", "Easter Sunday"),
            ("2023-04-17", "Easter Monday"),
            ("2023-04-18", "Easter Tuesday"),
            ("2023-05-01", "Labour Day"),
            ("2023-06-05", "Pentecost"),
            ("2023-08-15", "Assumption Day"),
            ("2023-10-01", "Cyprus Independence Day"),
            ("2023-10-28", "Ochi Day"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Day After Christmas"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-06", "Epiphany"),
            ("2023-02-27", "Green Monday"),
            ("2023-03-25", "Greek Independence Day"),
            ("2023-04-01", "Cyprus National Day"),
            ("2023-04-14", "Good Friday"),
            ("2023-04-15", "Holy Saturday"),
            ("2023-04-16", "Easter Sunday"),
            ("2023-04-17", "Easter Monday"),
            ("2023-04-18", "Easter Tuesday"),
            ("2023-05-01", "Labor Day"),
            ("2023-06-05", "Whit Monday"),
            ("2023-08-15", "Assumption Day"),
            ("2023-10-01", "Cyprus Independence Day"),
            ("2023-10-28", "Greek National Day"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Day After Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-01-06", "Богоявлення"),
            ("2023-02-27", "Чистий понеділок"),
            ("2023-03-25", "День незалежності Греції"),
            ("2023-04-01", "Національне свято Кіпру"),
            ("2023-04-14", "Страсна пʼятниця"),
            ("2023-04-15", "Велика субота"),
            ("2023-04-16", "Великдень"),
            ("2023-04-17", "Великодній понеділок"),
            ("2023-04-18", "Великодній вівторок"),
            ("2023-05-01", "День праці"),
            ("2023-06-05", "День Святого Духа"),
            ("2023-08-15", "Успіння Пресвятої Богородиці"),
            ("2023-10-01", "День незалежності Кіпру"),
            ("2023-10-28", "День Охі"),
            ("2023-12-24", "Святий вечір"),
            ("2023-12-25", "Різдво Христове"),
            ("2023-12-26", "Другий день Різдва"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.honduras import Honduras
from tests.common import CommonCountryTests


class TestHonduras(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Honduras)

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Sábado de Gloria"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_panamerican_day(self):
        self.assertHolidayName(
            "Día de las Américas", (f"{year}-04-14" for year in self.full_range)
        )

    def test_labor_day(self):
        self.assertHolidayName("Día del Trabajo", (f"{year}-05-01" for year in self.full_range))

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia", (f"{year}-09-15" for year in self.full_range)
        )

    def test_morazan_day(self):
        name = "Día de Morazán"
        self.assertHolidayName(name, (f"{year}-10-03" for year in range(self.start_year, 2015)))
        self.assertNoHolidayName(name, range(2015, self.end_year))

    def test_columbus_day(self):
        name = "Día de la Raza"
        self.assertHolidayName(name, (f"{year}-10-12" for year in range(self.start_year, 2015)))
        self.assertNoHolidayName(name, range(2015, self.end_year))

    def test_army_day(self):
        name = "Día de las Fuerzas Armadas"
        self.assertHolidayName(name, (f"{year}-10-21" for year in range(self.start_year, 2015)))
        self.assertNoHolidayName(name, range(2015, self.end_year))

    def test_morazan_weekend(self):
        name = "Semana Morazánica"
        self.assertHolidayName(
            name,
            "2020-10-07",
            "2020-10-08",
            "2020-10-09",
            "2021-10-06",
            "2021-10-07",
            "2021-10-08",
            "2022-10-05",
            "2022-10-06",
            "2022-10-07",
            "2023-10-04",
            "2023-10-05",
            "2023-10-06",
            "2024-10-02",
            "2024-10-03",
            "2024-10-04",
            "2025-10-01",
            "2025-10-02",
            "2025-10-03",
        )
        self.assertHolidayNameCount(name, 3, range(2015, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2015))

    def test_christmas_day(self):
        self.assertHolidayName("Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_2016(self):
        # https://www.officeholidays.com/countries/honduras/2016
        self.assertHolidayDatesInYear(
            2016,
            "2016-01-01",
            "2016-03-24",
            "2016-03-25",
            "2016-03-26",
            "2016-04-14",
            "2016-05-01",
            "2016-09-15",
            "2016-10-05",
            "2016-10-06",
            "2016-10-07",
            "2016-12-25",
        )

    def test_2021(self):
        # https://www.officeholidays.com/countries/honduras/2021
        self.assertHolidayDatesInYear(
            2021,
            "2021-01-01",
            "2021-04-01",
            "2021-04-02",
            "2021-04-03",
            "2021-04-14",
            "2021-05-01",
            "2021-09-15",
            "2021-10-06",
            "2021-10-07",
            "2021-10-08",
            "2021-12-25",
        )

    def test_2025(self):
        self.assertHolidayDatesInYear(
            2025,
            "2025-01-01",
            "2025-04-14",
            "2025-04-17",
            "2025-04-18",
            "2025-04-19",
            "2025-05-01",
            "2025-09-15",
            "2025-10-01",
            "2025-10-02",
            "2025-10-03",
            "2025-12-25",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-04-14", "Día de las Américas; Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-04-16", "Sábado de Gloria"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-09-15", "Día de la Independencia"),
            ("2022-10-05", "Semana Morazánica"),
            ("2022-10-06", "Semana Morazánica"),
            ("2022-10-07", "Semana Morazánica"),
            ("2022-12-25", "Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-14", "Maundy Thursday; Panamerican Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Holy Saturday"),
            ("2022-05-01", "Labor Day"),
            ("2022-09-15", "Independence Day"),
            ("2022-10-05", "Morazan Weekend"),
            ("2022-10-06", "Morazan Weekend"),
            ("2022-10-07", "Morazan Weekend"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-14", "Великий четвер; День Америки"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-16", "Велика субота"),
            ("2022-05-01", "День праці"),
            ("2022-09-15", "День незалежності"),
            ("2022-10-05", "Тиждень Морасана"),
            ("2022-10-06", "Тиждень Морасана"),
            ("2022-10-07", "Тиждень Морасана"),
            ("2022-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.san_marino import SanMarino
from tests.common import CommonCountryTests


class TestSanMarino(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SanMarino)

    def test_new_years_day(self):
        self.assertHolidayName("Capodanno", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        self.assertHolidayName("Epifania", (f"{year}-01-06" for year in self.full_range))

    def test_anniversary_of_liberation_of_republic(self):
        self.assertHolidayName(
            "Anniversario della Liberazione della Repubblica e Festa di Sant'Agata",
            (f"{year}-02-05" for year in self.full_range),
        )

    def test_anniversary_of_arengo(self):
        self.assertHolidayName(
            "Anniversario dell'Arengo", (f"{year}-03-25" for year in self.full_range)
        )

    def test_easter_sunday(self):
        name = "Pasqua"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Lunedì dell'angelo"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_investiture_of_captains_regent(self):
        self.assertHolidayName(
            "Investitura Capitani Reggenti",
            (f"{year}-04-01" for year in self.full_range),
            (f"{year}-10-01" for year in self.full_range),
        )

    def test_workers_day(self):
        self.assertHolidayName(
            "Festa dei lavoratori", (f"{year}-05-01" for year in self.full_range)
        )

    def test_corpus_christi(self):
        name = "Corpus Domini"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_anniversary_of_fall_of_fascism(self):
        self.assertHolidayName(
            "Anniversario della Caduta del Fascismo e Festa della Libertà",
            (f"{year}-07-28" for year in self.full_range),
        )

    def test_assumption_day(self):
        self.assertHolidayName(
            "Assunzione della B.V. Maria", (f"{year}-08-15" for year in self.full_range)
        )

    def test_saint_marinus_day(self):
        self.assertHolidayName(
            "San Marino, Anniversario di Fondazione della Repubblica",
            (f"{year}-09-03" for year in self.full_range),
        )

    def test_all_saints_day(self):
        self.assertHolidayName("Tutti i Santi", (f"{year}-11-01" for year in self.full_range))

    def test_commemoration_of_the_dead(self):
        self.assertHolidayName(
            "Commemorazione dei defunti", (f"{year}-11-02" for year in self.full_range)
        )

    def test_immaculate_conception(self):
        self.assertHolidayName(
            "Immacolata Concezione", (f"{year}-12-08" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Natale", (f"{year}-12-25" for year in self.full_range))

    def test_saint_stephens_day(self):
        self.assertHolidayName("Santo Stefano", (f"{year}-12-26" for year in self.full_range))

    def test_christmas_eve(self):
        name = "Vigilia di Natale"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Ultimo dell'anno"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Capodanno"),
            ("2023-01-06", "Epifania"),
            (
                "2023-02-05",
                "Anniversario della Liberazione della Repubblica e Festa di Sant'Agata",
            ),
            ("2023-03-25", "Anniversario dell'Arengo"),
            ("2023-04-01", "Investitura Capitani Reggenti"),
            ("2023-04-09", "Pasqua"),
            ("2023-04-10", "Lunedì dell'angelo"),
            ("2023-05-01", "Festa dei lavoratori"),
            ("2023-06-08", "Corpus Domini"),
            ("2023-07-28", "Anniversario della Caduta del Fascismo e Festa della Libertà"),
            ("2023-08-15", "Assunzione della B.V. Maria"),
            ("2023-09-03", "San Marino, Anniversario di Fondazione della Repubblica"),
            ("2023-10-01", "Investitura Capitani Reggenti"),
            ("2023-11-01", "Tutti i Santi"),
            ("2023-11-02", "Commemorazione dei defunti"),
            ("2023-12-08", "Immacolata Concezione"),
            ("2023-12-25", "Natale"),
            ("2023-12-26", "Santo Stefano"),
        )

    def test_2023_bank(self):
        self.assertBankHolidaysInYear(
            2023,
            ("2023-12-24", "Vigilia di Natale"),
            ("2023-12-31", "Ultimo dell'anno"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Capodanno"),
            ("2024-01-06", "Epifania"),
            (
                "2024-02-05",
                "Anniversario della Liberazione della Repubblica e Festa di Sant'Agata",
            ),
            ("2024-03-25", "Anniversario dell'Arengo"),
            ("2024-03-31", "Pasqua"),
            ("2024-04-01", "Investitura Capitani Reggenti; Lunedì dell'angelo"),
            ("2024-05-01", "Festa dei lavoratori"),
            ("2024-05-30", "Corpus Domini"),
            ("2024-07-28", "Anniversario della Caduta del Fascismo e Festa della Libertà"),
            ("2024-08-15", "Assunzione della B.V. Maria"),
            ("2024-09-03", "San Marino, Anniversario di Fondazione della Repubblica"),
            ("2024-10-01", "Investitura Capitani Reggenti"),
            ("2024-11-01", "Tutti i Santi"),
            ("2024-11-02", "Commemorazione dei defunti"),
            ("2024-12-08", "Immacolata Concezione"),
            ("2024-12-24", "Vigilia di Natale"),
            ("2024-12-25", "Natale"),
            ("2024-12-26", "Santo Stefano"),
            ("2024-12-31", "Ultimo dell'anno"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-06", "Epiphany"),
            (
                "2024-02-05",
                "Anniversary of the Liberation of the Republic and Feast of Saint Agatha",
            ),
            ("2024-03-25", "Anniversary of the Arengo"),
            ("2024-03-31", "Easter Sunday"),
            ("2024-04-01", "Easter Monday; Investiture of Captains Regent"),
            ("2024-05-01", "Workers' Day"),
            ("2024-05-30", "Corpus Christi"),
            ("2024-07-28", "Anniversary of the Fall of Fascism and Freedom Day"),
            ("2024-08-15", "Assumption Day"),
            ("2024-09-03", "Saint Marinus' Day, Anniversary of the Founding of the Republic"),
            ("2024-10-01", "Investiture of Captains Regent"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-11-02", "Commemoration of the Dead"),
            ("2024-12-08", "Immaculate Conception"),
            ("2024-12-24", "Christmas Eve"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Saint Stephen's Day"),
            ("2024-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-01-06", "Богоявлення"),
            ("2024-02-05", "Річниця визволення Республіки та День Святої Агати"),
            ("2024-03-25", "Річниця Аренго"),
            ("2024-03-31", "Великдень"),
            ("2024-04-01", "Інвеститура капітанів-регентів; Великодній понеділок"),
            ("2024-05-01", "День трудящих"),
            ("2024-05-30", "Свято Тіла і Крові Христових"),
            ("2024-07-28", "Річниця падіння фашизму та День свободи"),
            ("2024-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2024-09-03", "Річниця заснування Республіки та День Святого Марина"),
            ("2024-10-01", "Інвеститура капітанів-регентів"),
            ("2024-11-01", "День усіх святих"),
            ("2024-11-02", "День вшанування померлих"),
            ("2024-12-08", "Непорочне зачаття Діви Марії"),
            ("2024-12-24", "Святий вечір"),
            ("2024-12-25", "Різдво Христове"),
            ("2024-12-26", "День Святого Стефана"),
            ("2024-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.colombia import Colombia
from tests.common import CommonCountryTests


class TestColombia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Colombia)

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany_day(self):
        name = "Día de los Reyes Magos"
        self.assertNonObservedHolidayName(
            name, (f"{year}-01-06" for year in range(1951, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1951))
        obs_dts = (
            "2021-01-11",
            "2022-01-10",
            "2023-01-09",
            "2024-01-08",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_saint_josephs_day(self):
        name = "Día de San José"
        self.assertNonObservedHolidayName(
            name, (f"{year}-03-19" for year in range(1951, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1951))
        obs_dts = (
            "2020-03-23",
            "2021-03-22",
            "2022-03-21",
            "2023-03-20",
            "2024-03-25",
            "2025-03-24",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertHolidayName(name, range(1951, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1951))

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1951, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1951))

    def test_ascension_day(self):
        name = "Ascensión del señor"
        self.assertNonObservedHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertNonObservedHolidayName(name, range(1951, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1951))
        obs_dts = (
            "2020-05-25",
            "2021-05-17",
            "2022-05-30",
            "2023-05-22",
            "2024-05-13",
            "2025-06-02",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertNonObservedHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertNonObservedHolidayName(name, range(1951, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1951))
        obs_dts = (
            "2020-06-15",
            "2021-06-07",
            "2022-06-20",
            "2023-06-12",
            "2024-06-03",
            "2025-06-23",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_labor_day(self):
        self.assertHolidayName("Día del Trabajo", (f"{year}-05-01" for year in self.full_range))

    def test_sacred_heart(self):
        name = "Sagrado Corazón"
        self.assertNonObservedHolidayName(
            name,
            "2020-06-19",
            "2021-06-11",
            "2022-06-24",
            "2023-06-16",
            "2024-06-07",
            "2025-06-27",
        )
        self.assertNonObservedHolidayName(name, range(1984, self.end_year))
        self.assertNoNonObservedHolidayName(name, range(self.start_year, 1984))
        obs_dts = (
            "2020-06-22",
            "2021-06-14",
            "2022-06-27",
            "2023-06-19",
            "2024-06-10",
            "2025-06-30",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_saints_peter_and_paul_day(self):
        name = "San Pedro y San Pablo"
        self.assertNonObservedHolidayName(
            name, (f"{year}-06-29" for year in range(1951, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1951))
        obs_dts = (
            "2021-07-05",
            "2022-07-04",
            "2023-07-03",
            "2024-07-01",
            "2025-06-30",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia", (f"{year}-07-20" for year in self.full_range)
        )

    def test_battle_of_boyaca(self):
        self.assertHolidayName("Batalla de Boyacá", (f"{year}-08-07" for year in self.full_range))

    def test_columbus_day(self):
        name = "Día de la Raza"
        self.assertNonObservedHolidayName(name, (f"{year}-10-12" for year in self.full_range))
        obs_dts = (
            "2021-10-18",
            "2022-10-17",
            "2023-10-16",
            "2024-10-14",
            "2025-10-13",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_all_saints_day(self):
        name = "Día de Todos los Santos"
        self.assertNonObservedHolidayName(
            name, (f"{year}-11-01" for year in range(1951, self.end_year))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1951))
        obs_dts = (
            "2020-11-02",
            "2022-11-07",
            "2023-11-06",
            "2024-11-04",
            "2025-11-03",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_of_cartagena(self):
        name = "Independencia de Cartagena"
        self.assertNonObservedHolidayName(name, (f"{year}-11-11" for year in self.full_range))
        obs_dts = (
            "2020-11-16",
            "2021-11-15",
            "2022-11-14",
            "2023-11-13",
            "2025-11-17",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_immaculate_conception(self):
        name = "La Inmaculada Concepción"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(1951, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1951))

    def test_christmas_day(self):
        self.assertHolidayName("Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-01-10", "Día de los Reyes Magos (observado)"),
            ("2022-03-21", "Día de San José (observado)"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día del Trabajo"),
            ("2022-05-30", "Ascensión del señor (observado)"),
            ("2022-06-20", "Corpus Christi (observado)"),
            ("2022-06-27", "Sagrado Corazón (observado)"),
            ("2022-07-04", "San Pedro y San Pablo (observado)"),
            ("2022-07-20", "Día de la Independencia"),
            ("2022-08-07", "Batalla de Boyacá"),
            ("2022-08-15", "La Asunción"),
            ("2022-10-17", "Día de la Raza (observado)"),
            ("2022-11-07", "Día de Todos los Santos (observado)"),
            ("2022-11-14", "Independencia de Cartagena (observado)"),
            ("2022-12-08", "La Inmaculada Concepción"),
            ("2022-12-25", "Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-10", "Epiphany (observed)"),
            ("2022-03-21", "Saint Joseph's Day (observed)"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-30", "Ascension Day (observed)"),
            ("2022-06-20", "Corpus Christi (observed)"),
            ("2022-06-27", "Sacred Heart (observed)"),
            ("2022-07-04", "Saint Peter and Saint Paul's Day (observed)"),
            ("2022-07-20", "Independence Day"),
            ("2022-08-07", "Battle of Boyacá"),
            ("2022-08-15", "Assumption Day"),
            ("2022-10-17", "Columbus Day (observed)"),
            ("2022-11-07", "All Saints' Day (observed)"),
            ("2022-11-14", "Independence of Cartagena (observed)"),
            ("2022-12-08", "Immaculate Conception"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-10", "Богоявлення (вихідний)"),
            ("2022-03-21", "День Святого Йосипа (вихідний)"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-05-01", "День праці"),
            ("2022-05-30", "Вознесіння Господнє (вихідний)"),
            ("2022-06-20", "Свято Тіла і Крові Христових (вихідний)"),
            ("2022-06-27", "Свято Найсвятішого Серця Ісуса (вихідний)"),
            ("2022-07-04", "День Святих Петра і Павла (вихідний)"),
            ("2022-07-20", "День незалежності"),
            ("2022-08-07", "Річниця перемоги при Бояка"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-10-17", "День Колумба (вихідний)"),
            ("2022-11-07", "День усіх святих (вихідний)"),
            ("2022-11-14", "День незалежності Картахени (вихідний)"),
            ("2022-12-08", "Непорочне зачаття Діви Марії"),
            ("2022-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.morocco import Morocco
from tests.common import CommonCountryTests


class TestMorocco(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Morocco)

    def test_2019(self):
        self.assertHolidayDatesInYear(
            2019,
            "2019-01-01",
            "2019-01-11",
            "2019-05-01",
            "2019-06-04",
            "2019-06-05",
            "2019-07-30",
            "2019-08-11",
            "2019-08-12",
            "2019-08-14",
            "2019-08-20",
            "2019-08-21",
            "2019-08-31",
            "2019-11-06",
            "2019-11-09",
            "2019-11-10",
            "2019-11-18",
        )

    def test_1999(self):
        self.assertHolidayDatesInYear(
            1999,
            "1999-01-01",
            "1999-01-11",
            "1999-01-18",
            "1999-01-19",
            "1999-03-03",
            "1999-03-27",
            "1999-03-28",
            "1999-04-17",
            "1999-05-01",
            "1999-06-26",
            "1999-06-27",
            "1999-07-09",
            "1999-08-14",
            "1999-08-20",
            "1999-11-06",
            "1999-11-18",
        )

    def test_amazigh_new_year(self):
        self.assertHoliday("2024-01-13")
        self.assertNoHoliday("2023-01-13")

    def test_independence_manifesto_day(self):
        self.assertHoliday("1945-01-11")
        self.assertNoHoliday("1944-01-11")

    def test_independence_day(self):
        self.assertHolidayName("عيد الإستقلال", "1957-11-18")
        self.assertHolidayName("عيد العرش", "1956-11-18", "1957-11-18")

    def test_hijri_based(self):
        self.assertHoliday(
            # Eid al-Fitr
            "2021-05-13",
            "2021-05-14",
            # Eid al-Adha
            "2006-01-10",
            "2006-12-31",
            "2021-07-20",
            "2021-07-21",
            # Islamic New Year
            "2008-01-10",
            "2008-12-29",
            "2021-08-09",
            # Prophet Muhammad's Birthday
            "2021-10-18",
            "2021-10-19",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "رأس السنة الميلادية"),
            ("2023-01-11", "ذكرى تقديم وثيقة الاستقلال"),
            ("2023-04-21", "عيد الفطر (تقديري)"),
            ("2023-04-22", "عيد الفطر (تقديري)"),
            ("2023-05-01", "عيد العمال"),
            ("2023-06-28", "عيد الأضحى (تقديري)"),
            ("2023-06-29", "عيد الأضحى (تقديري)"),
            ("2023-07-19", "رأس السنة الهجرية (تقديري)"),
            ("2023-07-30", "عيد العرش"),
            ("2023-08-14", "ذكرى استرجاع إقليم وادي الذهب"),
            ("2023-08-20", "ذكرى ثورة الملك و الشعب"),
            ("2023-08-21", "عيد الشباب"),
            ("2023-09-27", "عيد المولد النبوي (تقديري)"),
            ("2023-09-28", "عيد المولد النبوي (تقديري)"),
            ("2023-11-06", "ذكرى المسيرة الخضراء"),
            ("2023-11-18", "عيد الإستقلال"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-11", "Proclamation of Independence Day"),
            ("2023-04-21", "Eid al-Fitr (estimated)"),
            ("2023-04-22", "Eid al-Fitr (estimated)"),
            ("2023-05-01", "Labor Day"),
            ("2023-06-28", "Eid al-Adha (estimated)"),
            ("2023-06-29", "Eid al-Adha (estimated)"),
            ("2023-07-19", "Islamic New Year (estimated)"),
            ("2023-07-30", "Throne Day"),
            ("2023-08-14", "Oued Ed-Dahab Day"),
            ("2023-08-20", "Revolution Day"),
            ("2023-08-21", "Youth Day"),
            ("2023-09-27", "Prophet's Birthday (estimated)"),
            ("2023-09-28", "Prophet's Birthday (estimated)"),
            ("2023-11-06", "Green March"),
            ("2023-11-18", "Independence Day"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2023-01-01", "Nouvel an"),
            ("2023-01-11", "Manifeste de l'indépendance"),
            ("2023-04-21", "Fête de la rupture du jeûne (estimé)"),
            ("2023-04-22", "Fête de la rupture du jeûne (estimé)"),
            ("2023-05-01", "Fête du Travail"),
            ("2023-06-28", "Fête du sacrifice (estimé)"),
            ("2023-06-29", "Fête du sacrifice (estimé)"),
            ("2023-07-19", "Nouvel an musulman (estimé)"),
            ("2023-07-30", "Fête du Trône"),
            ("2023-08-14", "Allégeance Oued Eddahab"),
            ("2023-08-20", "La révolution du roi et du peuple"),
            ("2023-08-21", "Fête de la Jeunesse"),
            ("2023-09-27", "Anniversaire du prophète (estimé)"),
            ("2023-09-28", "Anniversaire du prophète (estimé)"),
            ("2023-11-06", "La marche verte"),
            ("2023-11-18", "Fête de l'indépendance"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, AUG, SEP, OCT, NOV, DEC
from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import (
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
)
from holidays.holiday_base import HolidayBase


class India(
    HolidayBase, ChristianHolidays, HinduCalendarHolidays, InternationalHolidays, IslamicHolidays
):
    """India holidays.

    References:
        * <https://web.archive.org/web/20250413193616/https://www.india.gov.in/calendar>
        * <https://web.archive.org/web/20250413193624/https://www.india.gov.in/state-and-ut-holiday-calendar>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_India>
        * <https://web.archive.org/web/20250413193633/https://www.calendarlabs.com/holidays/india/2021>
        * <https://web.archive.org/web/20231118175007/http://slusi.dacnet.nic.in/watershedatlas/list_of_state_abbreviation.htm>
        * <https://web.archive.org/web/20231008063930/https://vahan.parivahan.gov.in/vahan4dashboard/>
        * Andaman and Nicobar Islands:
            * <https://web.archive.org/web/20251214133200/https://andamannicobar.gov.in/admin-pannel/othersdoc/1-29-Holiday%20List%202026%20Gazette.pdf>
        * Gujarat:
            * <https://web.archive.org/web/20260122052040/https://images-gujarati.indianexpress.com/2025/11/gujarat-government-Year-2026-holiday-list.pdf>
        * Kerala:
            * <https://web.archive.org/web/20260329164551/https://kerala.gov.in/showcalendar/2026>
        * Maharashtra:
            * <https://web.archive.org/web/20260327070656/https://www.mcgm.gov.in/irj/go/km/docs/documents/HomePage%20Data/Whats%20New/Public%20Holidays%202026.pdf>
        * Punjab:
            * <https://web.archive.org/web/20260216022835/https://punjab.gov.in/wp-content/uploads/2025/12/Calender-2026.pdf>
        * Tamil Nadu:
            * [Tamil Monthly Calendar](https://web.archive.org/web/20231228103352/https://www.tamildailycalendar.com/tamil_monthly_calendar.php)
            * [Tamil Calendar](https://web.archive.org/web/20250429125140/https://www.prokerala.com/general/calendar/tamilcalendar.php)
        * Telangana:
            * <https://web.archive.org/web/20260224050455/https://transport.telangana.gov.in/html/registration-districtcodes.html>
            * <https://web.archive.org/web/20250219131214/https://www.thehindu.com/news/national/telangana/cm-firm-on-having-states-identity-as-tg-not-ts/article68187923.ece>
    """

    country = "IN"
    default_language = "en_IN"
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    # India gained independence on August 15, 1947.
    start_year = 1948
    subdivisions = (
        "AN",  # Andaman and Nicobar Islands.
        "AP",  # Andhra Pradesh.
        "AR",  # Arunachal Pradesh (Arunāchal Pradesh).
        "AS",  # Assam.
        "BR",  # Bihar (Bihār).
        "CG",  # Chhattisgarh (Chhattīsgarh).
        "CH",  # Chandigarh (Chandīgarh).
        "DH",  # Dadra and Nagar Haveli and Daman and Diu(Dādra and Nagar Haveli and Damān and Diu)
        "DL",  # Delhi.
        "GA",  # Goa.
        "GJ",  # Gujarat (Gujarāt).
        "HP",  # Himachal Pradesh (Himāchal Pradesh).
        "HR",  # Haryana (Haryāna).
        "JH",  # Jharkhand (Jhārkhand).
        "JK",  # Jammu and Kashmir (Jammu and Kashmīr).
        "KA",  # Karnataka (Karnātaka).
        "KL",  # Kerala.
        "LA",  # Ladakh (Ladākh).
        "LD",  # Lakshadweep.
        "MH",  # Maharashtra (Mahārāshtra).
        "ML",  # Meghalaya (Meghālaya).
        "MN",  # Manipur.
        "MP",  # Madhya Pradesh.
        "MZ",  # Mizoram.
        "NL",  # Nagaland (Nāgāland).
        "OD",  # Odisha.
        "PB",  # Punjab.
        "PY",  # Puducherry.
        "RJ",  # Rajasthan (Rājasthān).
        "SK",  # Sikkim.
        "TN",  # Tamil Nadu (Tamil Nādu).
        "TR",  # Tripura.
        "TS",  # Telangana (Telangāna).
        "UK",  # Uttarakhand (Uttarākhand).
        "UP",  # Uttar Pradesh.
        "WB",  # West Bengal.
    )
    subdivisions_aliases = {
        "Andaman and Nicobar Islands": "AN",
        "Andhra Pradesh": "AP",
        "Arunachal Pradesh": "AR",
        "Arunāchal Pradesh": "AR",
        "Assam": "AS",
        "Bihar": "BR",
        "Bihār": "BR",
        "Chhattisgarh": "CG",
        "Chhattīsgarh": "CG",
        "Chandigarh": "CH",
        "Chandīgarh": "CH",
        "Dadra and Nagar Haveli and Daman and Diu": "DH",
        "Dādra and Nagar Haveli and Damān and Diu": "DH",
        "Delhi": "DL",
        "Goa": "GA",
        "Gujarat": "GJ",
        "Gujarāt": "GJ",
        "Himachal Pradesh": "HP",
        "Himāchal Pradesh": "HP",
        "Haryana": "HR",
        "Haryāna": "HR",
        "Jharkhand": "JH",
        "Jhārkhand": "JH",
        "Jammu and Kashmir": "JK",
        "Jammu and Kashmīr": "JK",
        "Karnataka": "KA",
        "Karnātaka": "KA",
        "Kerala": "KL",
        "Ladakh": "LA",
        "Ladākh": "LA",
        "Lakshadweep": "LD",
        "Maharashtra": "MH",
        "Mahārāshtra": "MH",
        "Meghalaya": "ML",
        "Meghālaya": "ML",
        "Manipur": "MN",
        "Madhya Pradesh": "MP",
        "Mizoram": "MZ",
        "Nagaland": "NL",
        "Nāgāland": "NL",
        "Odisha": "OD",
        "Punjab": "PB",
        "Puducherry": "PY",
        "Rajasthan": "RJ",
        "Rājasthān": "RJ",
        "Sikkim": "SK",
        "Tamil Nadu": "TN",
        "Tamil Nādu": "TN",
        "Tripura": "TR",
        "TG": "TS",
        "Telangana": "TS",
        "Telangāna": "TS",
        "Uttarakhand": "UK",
        "Uttarākhand": "UK",
        "Uttar Pradesh": "UP",
        "West Bengal": "WB",
    }
    supported_categories = (OPTIONAL, PUBLIC)
    supported_languages = ("bn", "en_IN", "en_US", "gu", "hi", "kn", "ml", "mr", "pa", "ta", "te")
    _deprecated_subdivisions = (
        "DD",  # Daman and Diu.
        "OR",  # Orissa.
    )

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.

        In India, the dates of the Islamic calendar usually fall a day later than
        the corresponding dates in the Umm al-Qura calendar.
        """
        ChristianHolidays.__init__(self)
        HinduCalendarHolidays.__init__(self)
        IslamicHolidays.__init__(
            self,
            cls=IndiaIslamicHolidays,
            show_estimated=islamic_show_estimated,
            calendar_delta_days=+1,
        )
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year >= 1950:
            # Republic Day.
            self._add_holiday_jan_26(tr("Republic Day"))

        # Independence Day.
        self._add_holiday_aug_15(tr("Independence Day"))

        # Gandhi Jayanti.
        self._add_holiday_oct_2(tr("Gandhi Jayanti"))

        # Hindu Holidays.
        if self._year < 2001 or self._year > 2035:
            warnings.warn(
                "Requested Holidays are available only from 2001 to 2035.",
                UserWarning,
                stacklevel=7,
            )

        # Buddha Purnima.
        self._add_buddha_purnima(tr("Buddha Purnima"))

        # Diwali.
        self._add_diwali_india(tr("Diwali"))

        # Janmashtami.
        self._add_janmashtami(tr("Janmashtami"))

        # Dussehra.
        self._add_dussehra(tr("Dussehra"))

        # Mahavir Jayanti.
        self._add_mahavir_jayanti(tr("Mahavir Jayanti"))

        # Maha Shivaratri.
        self._add_maha_shivaratri(tr("Maha Shivaratri"))

        # Guru Nanak Jayanti.
        self._add_guru_nanak_jayanti(tr("Guru Nanak Jayanti"))

        # Islamic holidays.

        # Ashura.
        self._add_ashura_day(tr("Muharram"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("Milad-un-Nabi"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("Id-ul-Fitr"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Bakrid"))

        # Christian holidays.

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Christmas.
        self._add_christmas_day(tr("Christmas"))

        if self.subdiv == "OR":
            self._populate_subdiv_od_public_holidays()

    def _populate_optional_holidays(self):
        # Hindu holidays.

        # Children's Day.
        self._add_holiday_nov_14(tr("Children's Day"))

        if self.subdiv != "MH":
            # Holi.
            self._add_holi(tr("Holi"))

        # Ganesh Chaturthi.
        self._add_ganesh_chaturthi(tr("Ganesh Chaturthi"))

        # Govardhan Puja.
        self._add_govardhan_puja(tr("Govardhan Puja"))

        # Labor Day.
        self._add_labor_day(tr("Labour Day"))

        # Maha Navami.
        self._add_maha_navami(tr("Maha Navami"))

        # Maharishi Valmiki Jayanti.
        self._add_maharishi_valmiki_jayanti(tr("Maharishi Valmiki Jayanti"))

        # Makar Sankranti.
        self._add_makar_sankranti(tr("Makar Sankranti"))

        # Raksha Bandhan.
        self._add_raksha_bandhan(tr("Raksha Bandhan"))

        # Ram Navami.
        self._add_ram_navami(tr("Ram Navami"))

        # Navratri / Sharad Navratri.
        self._add_sharad_navratri(tr("Navratri / Sharad Navratri"))

        # Christian holidays.

        # Easter Sunday.
        self._add_easter_sunday(tr("Easter Sunday"))

        # Palm Sunday.
        self._add_palm_sunday(tr("Palm Sunday"))

    # Andaman and Nicobar Islands.
    def _populate_subdiv_an_public_holidays(self):
        # Ram Navami.
        self._add_ram_navami(tr("Ram Navami"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Onam.
        self._add_onam(tr("Onam"))

    # Andhra Pradesh.
    def _populate_subdiv_ap_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Andhra Pradesh Foundation Day.
        self._add_holiday_nov_1(tr("Andhra Pradesh Foundation Day"))
        # Ugadi.
        self._add_gudi_padwa(tr("Ugadi"))

    # Assam.
    def _populate_subdiv_as_public_holidays(self):
        # Magh Bihu.
        self._add_makar_sankranti(tr("Magh Bihu"))
        # Assam Day.
        self._add_holiday_dec_2(tr("Assam Day"))

    # Bihar.
    def _populate_subdiv_br_public_holidays(self):
        # Chhath Puja.
        self._add_chhath_puja(tr("Chhath Puja"))
        # Bihar Day.
        self._add_holiday_mar_22(tr("Bihar Day"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))

    # Chandigarh.
    def _populate_subdiv_ch_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))

    # Chhattisgarh.
    def _populate_subdiv_cg_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Chhattisgarh Foundation Day.
        self._add_holiday_nov_1(tr("Chhattisgarh Foundation Day"))

    # Delhi.
    def _populate_subdiv_dl_public_holidays(self):
        # Chhath Puja.
        self._add_chhath_puja(tr("Chhath Puja"))

    # Goa.
    def _populate_subdiv_ga_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Goa Liberation Day.
        self._add_holiday_dec_19(tr("Goa Liberation Day"))

    # Gujarat.
    def _populate_subdiv_gj_public_holidays(self):
        # Makar Sankranti.
        self._add_makar_sankranti(tr("Uttarayan"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Gujarat Day.
        self._add_holiday_may_1(tr("Gujarat Day"))
        # Sardar Vallabhbhai Patel Jayanti.
        self._add_holiday_oct_31(tr("Sardar Vallabhbhai Patel Jayanti"))
        # Parsi New Year (Shahenshahi).
        self._add_parsi_new_year(tr("Parsi New Year (Shahenshahi)"))

    # Haryana.
    def _populate_subdiv_hr_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Haryana Foundation Day.
        self._add_holiday_nov_1(tr("Haryana Foundation Day"))

    # Himachal Pradesh.
    def _populate_subdiv_hp_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Himachal Day.
        self._add_holiday_apr_15(tr("Himachal Day"))

    # Jammu and Kashmir
    def _populate_subdiv_jk_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))

    # Jharkhand.
    def _populate_subdiv_jh_public_holidays(self):
        # Chhath Puja.
        self._add_chhath_puja(tr("Chhath Puja"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Jharkhand Formation Day.
        self._add_holiday_nov_15(tr("Jharkhand Formation Day"))

    # Karnataka.
    def _populate_subdiv_ka_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Karnataka Rajyotsav.
        self._add_holiday_nov_1(tr("Karnataka Rajyotsava"))
        # Ugadi.
        self._add_gudi_padwa(tr("Ugadi"))

    # Kerala.
    def _populate_subdiv_kl_public_holidays(self):
        # Onam.
        self._add_onam(tr("Onam"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Kerala Foundation Day.
        self._add_holiday_nov_1(tr("Kerala Foundation Day"))

    # Ladakh.
    def _populate_subdiv_la_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))

    # Maharashtra.
    def _populate_subdiv_mh_public_holidays(self):
        # Gudi Padwa.
        self._add_gudi_padwa(tr("Gudi Padwa"))
        # Chhatrapati Shivaji Maharaj Jayanti.
        self._add_holiday_feb_19(tr("Chhatrapati Shivaji Maharaj Jayanti"))

        holi_dates = {
            2026: (MAR, 3),
        }
        # Holi.
        name = tr("Holi")
        if dt := holi_dates.get(self._year):
            self._add_holiday(name, dt)
        else:
            self._add_holi(name)

        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Maharashtra Day.
        self._add_holiday_may_1(tr("Maharashtra Day"))
        # Parsi New Year (Shahenshahi).
        self._add_parsi_new_year(tr("Parsi New Year (Shahenshahi)"))

    # Madhya Pradesh.
    def _populate_subdiv_mp_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Madhya Pradesh Foundation Day.
        self._add_holiday_nov_1(tr("Madhya Pradesh Foundation Day"))

    # Mizoram.
    def _populate_subdiv_mz_public_holidays(self):
        # Mizoram State Day.
        self._add_holiday_feb_20(tr("Mizoram State Day"))

    # Nagaland.
    def _populate_subdiv_nl_public_holidays(self):
        # Nagaland State Inauguration Day.
        self._add_holiday_dec_1(tr("Nagaland State Inauguration Day"))

    # Orissa / Odisha.
    def _populate_subdiv_od_public_holidays(self):
        # Odisha Day.
        self._add_holiday_apr_1(tr("Odisha Day (Utkala Dibasa)"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Maha Vishuva Sankranti.
        self._add_holiday_apr_15(tr("Maha Vishuva Sankranti / Pana Sankranti"))

    # Puducherry.
    def _populate_subdiv_py_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Puducherry De Jure Transfer Day.
        self._add_holiday_aug_16(tr("Puducherry De Jure Transfer Day"))
        # Puducherry Liberation Day.
        self._add_holiday_nov_1(tr("Puducherry Liberation Day"))

    # Punjab.
    def _populate_subdiv_pb_public_holidays(self):
        # Guru Gobind Singh Jayanti.
        self._add_guru_gobind_singh_jayanti(tr("Guru Gobind Singh Jayanti"))
        # Vaisakhi.
        self._add_vaisakhi(tr("Vaisakhi"))
        # Lohri.
        self._add_holiday_jan_13(tr("Lohri"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # New Punjab Day.
        self._add_holiday_nov_1(tr("New Punjab Day"))

    # Rajasthan.
    def _populate_subdiv_rj_public_holidays(self):
        # Rajasthan Day.
        self._add_holiday_mar_30(tr("Rajasthan Day"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Maharana Pratap Jayanti.
        self._add_maharana_pratap_jayanti(tr("Maharana Pratap Jayanti"))

    # Sikkim.
    def _populate_subdiv_sk_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Sikkim State Day.
        self._add_holiday_may_16(tr("Sikkim State Day"))

    # Tamil Nadu.
    def _populate_subdiv_tn_public_holidays(self):
        # Pongal.
        self._add_pongal(tr("Pongal"))
        # Thiruvalluvar Day / Mattu Pongal.
        self._add_thiruvalluvar_day(tr("Thiruvalluvar Day / Mattu Pongal"))
        # Uzhavar Thirunal.
        self._add_uzhavar_thirunal(tr("Uzhavar Thirunal"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Puthandu.
        self._add_holiday_apr_14(tr("Puthandu (Tamil New Year)"))

    # Telangana.
    def _populate_subdiv_ts_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Telangana Formation Day.
        self._add_holiday_jun_2(tr("Telangana Formation Day"))
        # Bathukamma Festival.
        self._add_bathukamma(tr("Bathukamma Festival"))
        # Bonalu.
        self._add_bonalu(tr("Bonalu"))
        # Ugadi.
        self._add_gudi_padwa(tr("Ugadi"))

    # Uttarakhand.
    def _populate_subdiv_uk_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))

    # Uttar Pradesh.
    def _populate_subdiv_up_public_holidays(self):
        # Chhath Puja.
        self._add_chhath_puja(tr("Chhath Puja"))
        # UP Formation Day.
        self._add_holiday_jan_24(tr("UP Formation Day"))
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))

    # West Bengal.
    def _populate_subdiv_wb_public_holidays(self):
        # Dr. B. R. Ambedkar Jayanti.
        self._add_holiday_apr_14(tr("Dr. B. R. Ambedkar's Jayanti"))
        # Pohela Boisakh.
        self._add_holiday_apr_15(tr("Pohela Boishakh"))
        # Rabindra Jayanti.
        self._add_holiday_may_9(tr("Rabindra Jayanti"))


class IN(India):
    pass


class IND(India):
    pass


class IndiaIslamicHolidays(_CustomIslamicHolidays):
    ASHURA_DATES_CONFIRMED_YEARS = (2001, 2025)
    ASHURA_DATES = {
        2001: (APR, 4),
        2002: (MAR, 24),
        2005: (FEB, 19),
        2006: (FEB, 9),
        2008: (JAN, 19),
        2009: ((JAN, 7), (DEC, 28)),
        2021: (AUG, 20),
    }

    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_ADHA_DATES = {
        2005: (JAN, 21),
        2006: ((JAN, 11), (DEC, 31)),
        2007: (DEC, 20),
        2014: (OCT, 6),
        2015: (SEP, 25),
        2016: (SEP, 13),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2001, 2025)
    EID_AL_FITR_DATES = {
        2004: (NOV, 14),
        2005: (NOV, 3),
        2007: (OCT, 13),
        2010: (SEP, 10),
        2013: (AUG, 8),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2001, 2025)
    MAWLID_DATES = {
        2003: (MAY, 15),
        2004: (MAY, 3),
        2009: (MAR, 9),
        2015: ((JAN, 4), (DEC, 25)),
        2016: (DEC, 13),
        2017: (DEC, 2),
    }
